from pydantic_settings import BaseSettings
from pydantic import (
    BaseModel,
    AnyUrl,
    Field,
    field_validator
)
from typing import Any
from dotenv import load_dotenv
import json

class TrustedAuthority(BaseModel):
    """A trusted authority."""

    ISS: str
    JWKS_URI: str
    AUD: str | list[str] | None = None

    @field_validator("AUD")
    @classmethod
    def handle_list_in_aud(cls, v: str) -> str | list[str] | None:
        """Support AUD as list[str]."""
        if v and isinstance(v, str) and v.startswith("["):
            return json.loads(v)
        return v

from pydantic import ConfigDict
class SettingsConfigDict(ConfigDict, total=False):
    case_sensitive: bool
    env_prefix: str
    env_file: str | None
    env_file_encoding: str | None
    env_ignore_empty: bool
    env_nested_delimiter: str | None
    env_parse_none_str: str | None
    secrets_dir: str | str | None
    json_file: str | None
    json_file_encoding: str | None
    yaml_file: str | None
    yaml_file_encoding: str | None
    toml_file: str | None


class Settings(BaseSettings):
  title: str = "backend"
  description: str = "SLICES backend"

  api_url: str ="/apis/backend/v1alpha1"

  database_url: AnyUrl = "sqlite+aiosqlite:///example_async.db"  # Use SQLite with aiosqlite

  redis_db: str = "slices_db"
  redis_server: str = "host.docker.internal"
  redis_server_port: int = Field(6379, description="Redis server port number (0-65535)", ge=0, le=65535)

  SUPPORTED_JWT_SIGNING_ALGORITHMS: list[str] = ["ES256", "ES384", "EdDSA"]
  TRUSTED_AUTHORITIES: dict[str, TrustedAuthority] = {}

  JWT_LEEWAY: int = 15  # seconds

  model_config = SettingsConfigDict(
      env_file=(".env", ".env.secret"),
      env_file_encoding="utf-8",
      env_nested_delimiter="__",
      case_sensitive=True,
  )
  
# load_dotenv()
settings = Settings()